%% Parameters

% Field of view
FOV = [-12 12];

% Stimulus duration
stimulusDuration = 1250;
cropDuration = 250;

%% RFs, suppressive fields, preferred orientations and RF centers

% Make RFs, suppressive fields
[theta, deltaTheta, thetaPreferred, thetaRFs, ...
  xi, deltaXi, xiPreferred, xiRFs, ...
  Wyy, Wtheta, Wxi] = ...
  makeRFs(Ntheta, overlapTheta, overlapXi, FOV);
% Number of neurons
Nxi = length(xiPreferred);
N = Nxi * Ntheta;
% Number of stimulus samples
Mtheta = length(theta);
Mxi = length(xi);

% Redefine recurrent weights to be identity
Wyy = eye(Nxi);

% Fill b with b0
b = b0 * ones(1,Nxi,Ntheta);

% Adjust relative strength of surround suppression
Wxi = 0.5 * Wxi;

% Combine normalization weights
W = combineNormalizationWeights(Wtheta, Wxi);

%% Run simulations for each stimulus contrast and size

% Stimulus timing (msec)
t = [0:deltaT:stimulusDuration-deltaT]';
T = length(t);

% Initialize big matrix to hold the results
nContrasts = length(stimulusContrasts);
nSizes = length(stimulusSizes);
tSkip = floor(deltaTPlot / deltaT);
yAll = zeros(length(t(1:tSkip:end)), Nxi, Ntheta, 4, nSizes, nContrasts);

for cc = 1:nContrasts
  for ss = 1:nSizes
    contrast = stimulusContrasts(cc)
    stimulusSize = stimulusSizes(ss)
    
    % Make full contrast stimulus
    sTheta = zeros(Mtheta,1);
    if strcmp(stimulusPattern,'grating')
      sTheta(floor(Mtheta/2)+1) = 1;
    elseif strcmp(stimulusPattern,'plaid')
      sTheta(1) = 0.5;
      sTheta(floor(Mtheta/2)+1) = 0.5;
    elseif strcmp(stimulusPattern,'noise')
      sTheta = (1/Ntheta) * rand(Mtheta,1);
      sTheta = rand(Mtheta,1);
    end
    % Make sure it doesn't exceed 100% contrast
    sTheta = sTheta / sum(sTheta);
    
    % Stimulus size and center (convert to radians)
    sCenter = pi/180 * stimulusCenter;
    sSize = pi/180 * stimulusSize;
    sXi = zeros(Mxi,1);
    sXi(:) = abs(xi-sCenter) < sSize/2;

    % Input drive at full contrast
    z1 = (thetaRFs * sTheta) * (xiRFs * sXi)';
    
    % Input
    if strcmp(stimulusType,'static')
      x = contrast * ones(stimulusSize(t));
    else
      x = contrast .* (cos(2*pi*stimulusTF*t) + i*sin(2*pi*stimulusTF*t));
    end
    
    % Temporal prefilter
    xTmp = zeros(T,1,1);
    xTmp(:) = x;
    xPrefiltered = squeeze(recurrentQuadratureFilter(xTmp, lambdaX, omegaX, tauX, nFiltX, RIC, deltaT));
    xPrefiltered = xPrefiltered(:,nFiltX);

    % Input drive: multiply by z1 (input drive at full contrast), split into 4
    % phases from the real and imaginary parts
    z = zeros(T,Nxi,Ntheta,4);
    for iTheta = 1:Ntheta
      z(:,:,iTheta,1) = real(xPrefiltered) * z1(iTheta,:);
      z(:,:,iTheta,2) = -real(xPrefiltered) * z1(iTheta,:);
      z(:,:,iTheta,3) = imag(xPrefiltered) * z1(iTheta,:);
      z(:,:,iTheta,4) = -imag(xPrefiltered) * z1(iTheta,:);
    end

    % Run it
    if (variant == 3)
      [y, v, v1, a, u] = organicNormalizationOriX3(z, W, Wyy, tauV, tauA, tauU, omega, b, b0, sigma, deltaT);
    elseif (variant == 4)
      [y, v, v1, a, u] = organicNormalizationOriX4(z, W, Wyy, tauV, tauA, tauU, omega, b, b0, sigma, ca, deltaT);
    end
    yAll(:,:,:,:,ss,cc) = y(1:tSkip:end,:,:,:);
  end
end

% Subsample and save
deltaT = deltaTPlot;
t = t(1:tSkip:end);
T = length(t);
clear x xTmp xPrefiltered z y v v1 a u cc ss xiID iTheta contrast stimulusSize tSkip
save([figurePath,stimulusPattern,'Responses.mat'],'-v7.3');
