%% Parameters

parameters;
figurePath = 'Figures/Fig 5/'; 

% Field of view
FOV = [-2 2];

% Stimulus duration
preDuration = 100;
targetDuration = 250;

% Stimulus size and contrasts
stimulusSize = 1;
stimulusContrasts = [0.1 1.0];

%% Make RFs (orientation tuning curves and spatial RFs) and input stimulus

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Make RFs and spatial normalization weights %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Tuning curves are raised cosines
% Spatial RFs are raised cosines
% Recurrent weights are QMFs
% Spatial normalization weights are raised cosines (4x wider than RFs)
% Orientation normalization weights are 1s
[theta, deltaTheta, thetaPreferred, thetaRFs, ...
  xi, deltaXi, xiPreferred, xiRFs, ...
  Wyy, Wtheta, Wxi] = ...
  makeRFs(Ntheta, overlapTheta, overlapXi, FOV);

% Number of neurons
Nxi = length(xiPreferred);
N = Nxi * Ntheta;

% Number of stimulus samples
Mtheta = length(theta);
Mxi = length(xi);

% *** Redefine recurrent weights to be identity ***
Wyy = eye(Nxi);

% *** For debugging ***
% Wxi = eye(Nxi);

% Adjust relative strength of surround suppression
Wxi = 0.5 * Wxi;

% Combine normalization weights
W = combineNormalizationWeights(Wtheta, Wxi);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Make full contrast input stimulus and compute input drive %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Input stimulus is a matrix (Mtheta x Mxi), each element is the contrast
% of an orientation component at a spatial sample. The sum of the elements
% in the each column equals 1 at full contrast. Three different options for
% the stimulus pattern: single orientation component, random pair of
% orientations, random combination of all orientations.
% Option 1: single orientation component
% Option 2: orthogonal pair of orientations
% Option 3: random combination of orientations
sTheta = zeros(Mtheta,1);
if strcmp(stimulusPattern,'grating')
  sTheta(floor(Mtheta/2)+1) = 1;
elseif strcmp(stimulusPattern,'plaid')
  sTheta(1) = 0.5;
  sTheta(floor(Mtheta/2)+1) = 0.5;
elseif strcmp(stimulusPattern,'noise')
  sTheta = (1/Ntheta) * rand(Mtheta,1);
  sTheta = rand(Mtheta,1);
end
% Make sure it doesn't exceed 100% contrast
sTheta = sTheta / sum(sTheta);

% Stimulus size and center (convert to radians)
sCenter = pi/180 * stimulusCenter;
sSize = pi/180 * stimulusSize;
sXi = zeros(Mxi,1); 
sXi(:) = abs(xi-sCenter) < sSize/2;

% Input drive at full contrast
z1 = (thetaRFs * sTheta) * (xiRFs * sXi)';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Plot input stimulus, and input drive %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

figure(1); clf;

subplot(3,1,1)
plot(theta*180/pi, min(sTheta,1));
xlim([-90 90]);
set(gca,'xtick',[-90:30:90]);
ylim([0 1]);
set(gca,'ytick',[0:0.5:1]);
xlabel('Orientation');
title('Input stimulus');
drawPublishAxis

subplot(3,1,2)
plot(xi*180/pi, sXi);
xlim(FOV);
set(gca,'xtick',[FOV(1):1:FOV(2)]);
ylim([0 1]);
set(gca,'ytick',[0:0.5:1]);
xlabel('Spatial position (deg visual angle)');
drawPublishAxis

subplot(3,1,3)
plot(xiPreferred*180/pi, z1');
xlim(FOV);
set(gca,'xtick',[FOV(1):1:FOV(2)]);
ylim([0 1]);
set(gca,'ytick',[0:0.5:1]);
xlabel('RF center (deg visual angle)');
title('Input drive');
drawPublishAxis

print([figurePath,'/inputDrive.eps'],'-depsc','-painters');

%% Dynamics of responses

% Attentional gain: optionally boost input drive
attentionGain = 1;
%attentionGain = 1.2;
b = b0 * ones(1,Nxi,Ntheta);

% Stimulus timing (msec)
totalDuration = 2 * targetDuration * length(stimulusContrasts);
t = [-preDuration:deltaT:totalDuration-deltaT]';
T = length(t);

% Input
% x0 is sequence of increasing contrasts (modulator)
% x is multiplied by (co-)sinusoidal modulation (carrier)
x0 = zeros(T,1);
for ii = 1:length(stimulusContrasts)
  targetOnsetTime = 2*(ii-1) * targetDuration;
  targetOffsetTime = (2*(ii-1)+1) * targetDuration;
  id = (t>targetOnsetTime) & (t<targetOffsetTime);
  x0(id) = stimulusContrasts(ii);
end
if strcmp(stimulusType,'static')
  x = x0;
else
  x = x0 .* (cos(2*pi*stimulusTF*t) + i*sin(2*pi*stimulusTF*t));
end

% Temporal prefilter
xTmp = zeros(T,1,1);
xTmp(:) = x;
xPrefiltered = squeeze(recurrentQuadratureFilter(xTmp, lambdaX, omegaX, tauX, nFiltX, RIC, deltaT));
xPrefiltered = xPrefiltered(:,nFiltX);

% Input drive: multiply by z1 (input drive at full contrast), split into 4
% phases from the real and imaginary parts
z = zeros(T,Nxi,Ntheta,4);
for iTheta = 1:Ntheta
  z(:,:,iTheta,1) = real(xPrefiltered) * z1(iTheta,:);
  z(:,:,iTheta,2) = -real(xPrefiltered) * z1(iTheta,:);
  z(:,:,iTheta,3) = imag(xPrefiltered) * z1(iTheta,:);
  z(:,:,iTheta,4) = -imag(xPrefiltered) * z1(iTheta,:);
end

%%%%%%%%%%
% Run it %
%%%%%%%%%%

if (variant == 3)
  [y, v, v1, a, u] = organicNormalizationOriX3(z, W, Wyy, tauV, tauA, tauU, omega, b, b0, sigma, deltaT);
elseif (variant == 4)
  [y, v, v1, a, u] = organicNormalizationOriX4(z, W, Wyy, tauV, tauA, tauU, omega, b, b0, sigma, ca, deltaT);
end

% Responses of neurons with RFs centered at stimulus center
xiID = find(abs(xiPreferred-sCenter) == min(abs(xiPreferred-sCenter)));
zc = squeeze(z(:,xiID,:,:));
yc = squeeze(y(:,xiID,:,:));
ySum = sum(yc,3);
ac = squeeze(a(:,xiID,:,:));
uc = squeeze(u(:,xiID,:,:));
vc = squeeze(v(:,xiID,:,:));
v1c = squeeze(v1(:,xiID,:,:));

% Effective feedforward gain (ignoring attention gain)
ge = ((1+ac)./ac).^2 * (b0/(1+b0))^2;

%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compute steady state responses (ignoring attention gain) for comparison %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if (variant == 3)
  [zSteadyState, ySteadyState, aSteadyState, uSteadyState] = organicNormalizationOriXsteadyState3(z1, stimulusContrasts, W, b0, sigma);
elseif (variant == 4)
  [zSteadyState, ySteadyState, aSteadyState, uSteadyState] = organicNormalizationOriXsteadyState4(z1, stimulusContrasts, W, b0, sigma);
end

zSteadyStateC = zSteadyState(:,xiID,:);
ySteadyStateC = ySteadyState(:,xiID,:);
ySteadyStateSum = sum(ySteadyStateC,3);
aSteadyStateC = aSteadyState(:,xiID,:);
uSteadyStateC = uSteadyState(:,xiID,:);

%%
%%%%%%%%%%%
% Plot it %
%%%%%%%%%%%

% Subsample time for plotting
tSkip = floor(deltaTPlot / deltaT);

figure(2); clf;

zScale = max(max(max(zSteadyState)));
yScale = attentionGain * max(max(max(ySteadyState)));
ySumScale = attentionGain * max(max(ySteadyStateSum));
aSumScale = max(sum(aSteadyStateC,3));
uSumScale = max(sum(sqrt(uSteadyStateC),3));

subplot(4,1,1);
zAmp = squeeze(sqrt(zc(:,:,1).^2 + zc(:,:,3).^2));
plot(t(1:tSkip:end),[zc((1:tSkip:end),:,1) zc((1:tSkip:end),:,2) zc((1:tSkip:end),:,3) zc((1:tSkip:end),:,4)]);
hold on
for cc = 1:length(stimulusContrasts);
  targetTime = (2*(cc-1)+1) * targetDuration;
  ss = zSteadyStateC(cc,1,:);
  plot(repmat(targetTime,2,Ntheta),repmat(ss(:)',2,1),'o');
  plot(repmat(targetTime,2,Ntheta),repmat(-ss(:)',2,1),'o');
end
hold off
xlim([-preDuration,totalDuration]);
set(gca,'xtick',[0:targetDuration:totalDuration]);
ylim([-zScale zScale]);
xlabel('Time (msec)');
title('Input drive: z');
drawPublishAxis

subplot(4,1,2);
plot(t(1:tSkip:end),[yc((1:tSkip:end),:,1) yc((1:tSkip:end),:,2) yc((1:tSkip:end),:,3) yc((1:tSkip:end),:,4)]);
hold on
for cc = 1:length(stimulusContrasts);
  targetTime = (2*(cc-1)+1) * targetDuration;
  ss = ySteadyStateC(cc,1,:);
  plot(repmat(targetTime,2,Ntheta),repmat(ss(:)',2,1),'o');
end
hold off
xlim([-preDuration,totalDuration]);
set(gca,'xtick',[0:targetDuration:totalDuration]);
ylim([0 2*yScale]);
set(gca,'Clipping','off');
xlabel('Time (msec)');
title('Responses: simple cells');
drawPublishAxis

subplot(4,1,3);
plot(t(1:tSkip:end),ySum(1:tSkip:end,:));
hold on
for cc = 1:length(stimulusContrasts);
  targetTime = (2*(cc-1)+1) * targetDuration;
  ss = ySteadyStateC(cc,1,:);
  plot(repmat(targetTime,2,Ntheta),repmat(ss(:)',2,1),'o');
end
hold off
xlim([-preDuration,totalDuration]);
set(gca,'xtick',[0:targetDuration:totalDuration]);
ylim([0 2*yScale]);
set(gca,'Clipping','off');
xlabel('Time (msec)');
title('Responses: complex cells');
drawPublishAxis

subplot(4,1,4);
ySumAll = sum(ySum,2);
plot(t(1:tSkip:end),ySumAll(1:tSkip:end),'k');
hold on
plot(t(1:tSkip:end),sum(sum(y(1:tSkip:end,:,:,:),3),4));
for cc = 1:length(stimulusContrasts);
  targetTime = (2*(cc-1)+1) * targetDuration;
  targetTime = (2*(cc-1)+1) * targetDuration;
  plot(targetTime,ySteadyStateSum(cc),'ok');
end
plot(t,0.5*ones(size(t)),'k-.');
plot(t,ones(size(t)),'k-.');
hold off
xlim([-preDuration,totalDuration]);
set(gca,'xtick',[0:targetDuration:totalDuration]);
ylim([0 2*ySumScale]);
set(gca,'Clipping','off');
xlabel('Time (msec)');
title('Summed responses for each RF center');
drawPublishAxis

print([figurePath,'/dynamics1.eps'],'-depsc','-painters');

figure(3); clf;

subplot(5,1,1);
plot(t(1:tSkip:end),ac(1:tSkip:end,:));
xlim([-preDuration,totalDuration]);
set(gca,'xtick',[0:targetDuration:totalDuration]);
set(gca,'Clipping','off');
xlabel('Time (msec)');
title('Modulator: a');
drawPublishAxis

subplot(5,1,2);
plot(t(1:tSkip:end),sqrt(uc(1:tSkip:end,:)));
xlim([-preDuration,totalDuration]);
set(gca,'xtick',[0:targetDuration:totalDuration]);
set(gca,'Clipping','off');
xlabel('Time (msec)');
title('Modulator: sqrt(u)');
drawPublishAxis

subplot(5,1,3);
semilogy(t(1:tSkip:end),ge(1:tSkip:end,:));
xlim([-preDuration,totalDuration]);
set(gca,'xtick',[0:targetDuration:totalDuration]);
set(gca,'Clipping','off');
xlabel('Time (msec)');
title('Effective gain: ge');
drawPublishAxis

subplot(5,1,4);
plot(t(1:tSkip:end),vc((1:tSkip:end),:,1));
xlim([-preDuration,totalDuration]);
set(gca,'xtick',[0:targetDuration:totalDuration]);
ylim([-sqrt(yScale) sqrt(yScale)]);
set(gca,'Clipping','off');
xlabel('Time (msec)');
title('Membrane potential responses: v');
drawPublishAxis

subplot(5,1,5);
plot(t(1:tSkip:end),v1c((1:tSkip:end),:,1));
xlim([-preDuration,totalDuration]);
set(gca,'xtick',[0:targetDuration:totalDuration]);
ylim(b0*[-zScale zScale]);
set(gca,'Clipping','off');
xlabel('Time (msec)');
title('Membrane potential responses w/ feedback disabled');
drawPublishAxis

print([figurePath,'/dynamics2.eps'],'-depsc','-painters');

% Plot relative phase of y, a, and u
figure(4); clf;
aSum = sum(ac,2);
uSum = sum(sqrt(uc),2);
plot(t(1:tSkip:end),[ySumAll(1:tSkip:end)/ySumScale aSum(1:tSkip:end)/aSumScale uSum(1:tSkip:end)/uSumScale]);
hold on
plot(t,ones(size(t)),'k-.');
hold off
xlim([2*targetDuration*(length(stimulusContrasts)-1),2*targetDuration*length(stimulusContrasts)]);
xlabel('Time (msec)');
title('Relative response phase of y, a, and u');
drawPublishAxis

print([figurePath,'/phaseAlignment.eps'],'-depsc','-painters');

%%

% Plot delayed onset with distance between target and RF center
nC = length(stimulusContrasts);
yiSteadyState = sum(ySteadyState,3);
yiSum = sum(sum(y,3),4);
yiSumScaled = zeros(size(yiSum));
for cc = 1:nC
  tID = (t >= 2*targetDuration*(cc-1)) & (t < 2*targetDuration*cc);
  for ii = 1:length(xiPreferred)
    if (yiSteadyState(cc,ii) > 1e-6)
      yiSumScaled(tID,ii) = yiSum(tID,ii) / yiSteadyState(cc,ii);
    end
  end
end

ii = find(xiPreferred == 0);
yiSScenter = yiSteadyState(:,ii)

figure(5); clf;

% Change color map
plotID = find(max(yiSum) > 1e-4);
nCurves = length(plotID);
cmapHot = colormap(hot);
cmapHot = cmapHot(1:42,:);
cmapHotFlip = cmapHot(end:-1:1,:);
cmapTmp = [cmapHotFlip; [0,0,0]; cmapHot];
nColors = size(cmapTmp,1);
cmap = zeros(length(plotID),3);
for ii = 1:3
  cmap(:,ii) = interp1(linspace(-1,1,nColors)', cmapTmp(:,ii), linspace(-1,1,nCurves)');
end
set(groot,'defaultAxesColorOrder',cmap);

subplot(2,1,1)
ts = t(1:tSkip:end);
tSkipSize = size(ts);
plot(ts, yiSum(1:tSkip:end,plotID));
hold on
for cc = 1:nC
  plot(ts,yiSScenter(cc)*ones(tSkipSize),'k-.');
end
hold off
xlim([-preDuration,totalDuration]);
set(gca,'xtick',[0:targetDuration:totalDuration]);
xlabel('Time (msec)');
title('Relative timing of responses for each RF center');
drawPublishAxis

subplot(2,1,2)
plot(t(1:tSkip:end), yiSumScaled(1:tSkip:end,plotID));
hold on
plot(t,ones(size(t)),'k-.');
hold off
xlim([-preDuration,totalDuration]);
set(gca,'xtick',[0:targetDuration:totalDuration]);
xlabel('Time (msec)');
title('Relative timing of responses for each RF center');
drawPublishAxis

print([figurePath,'/delayedOnset.eps'],'-depsc','-painters');

% Restore color map
set(groot,'defaultAxesColorOrder',defaultColors);
