function [theta, deltaTheta, thetaPreferred, thetaRFs, ...
          xi, deltaXi, xiPreferred, xiRFs, ...
          Wyy, Wtheta, Wxi] = ...
          makeRFs(Ntheta, overlapTheta, overlapXi, FOV)
%
% Ntheta: number of orientation bands
% overlapTheta: overlap of orientation turning curves
% overlapXi: overlap of spatial RFs
% FOV: field of view of spatial RFs
%
% theta: orientation samples
% deltaTheta: orientation sample spacing
% thetaPreferred: preferred orientations
% thetaRFs: tuning curves
% xi: spatial samples
% deltaXi: spatial sample spacing
% xiPreferred: RF centers
% xiRFs: spatial RFs
% Wyy: recurrent weight matrix
% W: normalization weights (combined from Wtheta and Wxi)
% Wtheta: orientation component of normalization weights
% Wxi: spatial component of normalization weights
%
% DJH, 8/2018
% Copyright (c) 2019, David J. Heeger, New York University

% Number of orientation bands and orientation sampling.
deltaTheta = pi/(10*Ntheta);

% Number of RFs and spatial sampling
Nxi = 100 * overlapXi; 
deltaXi = pi/(10*Nxi);

% Total number of neurons
N = Nxi * Ntheta;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Orientation tuning curves %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

theta = [-pi/2:deltaTheta:pi/2-deltaTheta];
Mtheta = length(theta);

spacingTheta = pi/Ntheta;
thetaPreferred = linspace(-pi/2,pi/2-spacingTheta,Ntheta);

thetaSc = theta * (Ntheta/4) / overlapTheta;
rf = cos(thetaSc).^2;
id = (thetaSc > -pi/2) & (thetaSc < pi/2);
rf = rf .* id;
thetaRFs = zeros(Ntheta,Mtheta);
for ii = 1:Ntheta
  sh = -(Mtheta/2)+(ii-1)*Mtheta/Ntheta;
  thetaRFs(ii,:) = circshift(rf,[0,sh]);
end
sumThetaRFs = sqrt(mean(sum(thetaRFs.^2)));
thetaRFs = thetaRFs / sumThetaRFs;

%%%%%%%%%%%%%%
% Spatial RF %
%%%%%%%%%%%%%%

xi = [-pi:deltaXi:pi-deltaXi];
Mxi = length(xi);

spacingXi = 2*pi/Nxi;
xiPreferred = linspace(-pi,pi-spacingXi,Nxi);

xiSc = xi * (Nxi/8) / overlapXi;
rf = cos(xiSc).^2;
id = (xiSc > -pi/2) & (xiSc < pi/2);
rf = rf .* id;
xiRFs = zeros(Nxi,Mxi);
for ii = 1:Nxi
  sh = -(Mxi/2)+(ii-1)*Mxi/Nxi;
  xiRFs(ii,:) = circshift(rf,[0,sh]);
end
% sumXiRFs = sqrt(mean(sum(xiRFs.^2)));
sumXiRFs =  mean(sum(xiRFs,2));
xiRFs = xiRFs / sumXiRFs;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Spatial normalization weights %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

overlap = 4*overlapXi;
xiPrefSc = xiPreferred * (Nxi/8) / overlap;
rf = cos(xiPrefSc).^2;
id = (xiPrefSc > -pi/2) & (xiPrefSc < pi/2);
rf = rf .* id;
Wxi = zeros(Nxi,Nxi);
for ii = 1:Nxi
  sh = -(Nxi/2)+(ii-1);
  Wxi(ii,:) = circshift(rf,[0,sh]);
end

% Compensate for RF overlap. 
Wxi = Wxi / overlapXi;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Orientation normalization weights %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Normalization weights are an NxN matrix. Two different options: even sum
% over all orientation, random weights. Comment/uncomment below to pick
% one.

% Option 1: sum evenly over all orientations
Wtheta = ones(Ntheta,Ntheta);

% Option 2: random symmetric weight matrix.
% tmp = rand(Ntheta,Ntheta);
% Wtheta = tmp' * tmp;
% Wtheta = Wtheta / (mean(mean(Wtheta)));

% Option 3: random  weight matrix.
% Wtheta = rand(Ntheta,Ntheta);
% Wtheta = Wtheta / (mean(mean(Wtheta)));

%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Recurrent weight matrix %
%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Make convolutional, symmetric matrix (i.e., with real-valued
% eigenvectors), and with a relatively large number of eigenvalues equal to
% 1. Start with qmf filter, and refine it so that half(+1) of the
% eigenvalues = 1.

% Make cicular convolution matrix with kernel
kernel = namedFilter('qmf9');
K = length(kernel);
convMat = zeros(Nxi,Nxi);
for row = 1:Nxi
  k = 1;
  for col = row-floor(K/2):row+floor(K/2)
    convMat(row,rem(col-1+Nxi,Nxi)+1) = kernel(k);
    k = k + 1;
  end
end

% Force half(+1) of the eigenvalues to be = 1.
[Utmp,Stmp,Vtmp] = svd(convMat);
Stmp = diag(Stmp);
Stmp(Stmp>=1) = 1;

% Optionally force the other half of the eigenvalues to be 0.
% Stmp(Stmp<1) = 0;

% Construct final weight matrix by composing eigenvectors and modulated
% eigenvalues.
Wyy = Vtmp * diag(Stmp) * Vtmp';

% Get dimensionality
Wd = eig(Wyy);
id = find(abs(real(Wd)-1.0)<1e-12);
d = length(id);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Warp space by cortical magnification %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% lambda = 1.0;
lambda = 0.9;
xi = pi * sign(xi) .* (lambda*(abs(xi)/pi).^(3.5) + (1-lambda)*(abs(xi)/pi));
xiPreferred = pi * sign(xiPreferred) .* (lambda*(abs(xiPreferred)/pi).^(3.5) + (1-lambda)*(abs(xiPreferred)/pi));

%%%%%%%%%%%%%%%%%%%%%%%%%%
% Restrict field of view %
%%%%%%%%%%%%%%%%%%%%%%%%%%

FOV = FOV*pi/180;
ID = (xiPreferred >= FOV(1)) & (xiPreferred <= FOV(2));
xiPreferred = xiPreferred(ID);
xiRFs = xiRFs(ID,:);
Wyy = Wyy(ID,ID);
Wxi = Wxi(ID,ID);
Nxi = length(xiPreferred);

return
