function [z, y, v, yd, vd, n, a, u] = organicNormalizationFull(x, numOrientations, bandwidth, tauV, tauA, tauU, omega, b0, sigma, deltaT)
%
% DJH, 8/2018
% Copyright (c) 2019, David J. Heeger, New York University

% Dimensions
dims = [size(x,2) size(x,3)];
T = size(x,1);

% Make frequency responses
numLevels = maxLevel(dims,bandwidth);
pyrFRs = makeQuadSteerFRs(dims,numLevels,numOrientations,bandwidth);

% Limits
uMin = ((sigma*b0)/(1+b0)).^2;
aMin = sqrt(uMin)./(1-sqrt(uMin));

% Initialize output and temporary variables
z = zeros([T dims numLevels numOrientations 2 4]);

y = zeros(size(z));
yHat = zeros(size(z));
rootY = zeros(size(z));
v = zeros(size(z));
deltaV = zeros(size(z));

yd = zeros(size(z));
rootYd = zeros(size(z));
vd = zeros(size(z));

aTmp = zeros(size(z));
n = zeros(size(x));
a = zeros(size(x)); a(1,:,:) = aMin;
deltaA = zeros(size(x));
u = zeros(size(x)); u(1,:,:) = uMin;
deltaU = zeros(size(x));
xTmp = zeros(dims);

evenFast = zeros([dims numLevels numOrientations]);
evenSlow = zeros([dims numLevels numOrientations]);
oddFast = zeros([dims numLevels numOrientations]);
oddSlow = zeros([dims numLevels numOrientations]);

% Run it
h = waitbar(0,'Computing...');
hStep = round(T/100);
for tt = 1:T-1
  if (round(tt/hStep) == tt/hStep)
    waitbar(tt/T,h);
  end
  
  % Compute z
  % Spatial pyramids for both real and imaginary parts of the temporal
  % filter.
  xTmp(:,:) = real(x(tt,:,:));
  pyrReal = buildQuadSteerPyr(xTmp,pyrFRs);
  xTmp(:,:) = imag(x(tt,:,:));
  pyrImag = buildQuadSteerPyr(xTmp,pyrFRs);
  
  z(tt,:,:,:,:,1,1) = real(pyrReal{3});
  z(tt,:,:,:,:,1,2) = -real(pyrReal{3});
  z(tt,:,:,:,:,1,3) = real(pyrImag{3});
  z(tt,:,:,:,:,1,4) = -real(pyrImag{3});
  z(tt,:,:,:,:,2,1) = imag(pyrReal{3});
  z(tt,:,:,:,:,2,2) = -imag(pyrReal{3});
  z(tt,:,:,:,:,2,3) = imag(pyrImag{3});
  z(tt,:,:,:,:,2,4) = -imag(pyrImag{3});

  % Normalize by summing across all spatial frequencies, orientations,
  % directions, phases
  n(tt,:,:) = sum(sum(sum(sum(y(tt,:,:,:,:,:,:),7),6),5),4) .* u(tt,:,:,:,:,:,:);
    
  % Compute u
  deltaU(tt,:,:) = (deltaT/tauU) * (-u(tt,:,:) + n(tt,:,:) + uMin);
  u(tt+1,:,:) = u(tt,:,:) + deltaU(tt,:,:);
  
  % Compute a
  deltaA(tt,:,:) = (deltaT/tauA) * (-a(tt,:,:) + sqrt(u(tt,:,:)) + sqrt(u(tt,:,:)).*a(tt,:,:));
  a(tt+1,:,:) = a(tt,:,:) + deltaA(tt,:,:);
  for ll = 1:numLevels
    for oo = 1:numOrientations
      for dd = 1:2
        for pp = 1:4
          aTmp(tt+1,:,:,ll,oo,dd,pp) = a(tt+1,:,:);
        end
      end
    end
  end
  
  % Compute yHat
  yHat(tt,:,:,:,:,:,1) = (rootY(tt,:,:,:,:,:,1)-rootY(tt,:,:,:,:,:,2)) - 2*pi*omega*tauV*(rootY(tt,:,:,:,:,:,3)-rootY(tt,:,:,:,:,:,4));
  yHat(tt,:,:,:,:,:,2) = (rootY(tt,:,:,:,:,:,2)-rootY(tt,:,:,:,:,:,1)) - 2*pi*omega*tauV*(rootY(tt,:,:,:,:,:,4)-rootY(tt,:,:,:,:,:,3));
  yHat(tt,:,:,:,:,:,3) = 2*pi*omega*tauV*(rootY(tt,:,:,:,:,:,1)-rootY(tt,:,:,:,:,:,2)) + (rootY(tt,:,:,:,:,:,3)-rootY(tt,:,:,:,:,:,4));
  yHat(tt,:,:,:,:,:,4) = 2*pi*omega*tauV*(rootY(tt,:,:,:,:,:,2)-rootY(tt,:,:,:,:,:,1)) + (rootY(tt,:,:,:,:,:,4)-rootY(tt,:,:,:,:,:,3));
 
  % Compute v
  deltaV(tt,:,:,:,:,:,:) = (deltaT/tauV) * (-v(tt,:,:,:,:,:,:) + ...
                           (b0./(1+b0)) .* z(tt,:,:,:,:,:,:) + ...
                           (1./(1+aTmp(tt,:,:,:,:,:,:))) .* yHat(tt,:,:,:,:,:,:) );
  v(tt+1,:,:,:,:,:,:) = v(tt,:,:,:,:,:,:) + deltaV(tt,:,:,:,:,:,:);
  
  % Compute y
  y(tt+1,:,:,:,:,:,:) = max(v(tt+1,:,:,:,:,:,:),0).^2;
  rootY(tt+1,:,:,:,:,:,:) = sqrt(y(tt+1,:,:,:,:,:,:));
  
  % Recombine (Adelson & Bergen) to get direction-selective responses.  
  evenFast(:,:,:,:) = rootY(tt+1,:,:,:,:,1,1) - rootY(tt+1,:,:,:,:,1,2);
  evenSlow(:,:,:,:) = rootY(tt+1,:,:,:,:,1,3) - rootY(tt+1,:,:,:,:,1,4);
  oddFast(:,:,:,:) = rootY(tt+1,:,:,:,:,2,1) - rootY(tt+1,:,:,:,:,2,2);
  oddSlow(:,:,:,:) = rootY(tt+1,:,:,:,:,2,3) - rootY(tt+1,:,:,:,:,2,4);
  % *** Without square root: introduces some distortion ***
  %   evenFast(:,:,:,:) = y(tt+1,:,:,:,:,1,1) - y(tt+1,:,:,:,:,1,2);
  %   evenSlow(:,:,:,:) = y(tt+1,:,:,:,:,1,3) - y(tt+1,:,:,:,:,1,4);
  %   oddFast(:,:,:,:) = y(tt+1,:,:,:,:,2,1) - y(tt+1,:,:,:,:,2,2);
  %   oddSlow(:,:,:,:) = y(tt+1,:,:,:,:,2,3) - y(tt+1,:,:,:,:,2,4);
  
  vd(tt+1,:,:,:,:,1,1) = (-oddFast + evenSlow);
  vd(tt+1,:,:,:,:,1,2) = (oddSlow + evenFast);
  vd(tt+1,:,:,:,:,1,3) = -(-oddFast + evenSlow);
  vd(tt+1,:,:,:,:,1,4) = -(oddSlow + evenFast);
  vd(tt+1,:,:,:,:,2,1) = (oddFast + evenSlow);
  vd(tt+1,:,:,:,:,2,2) = (-oddSlow + evenFast);
  vd(tt+1,:,:,:,:,2,3) = -(oddFast + evenSlow);
  vd(tt+1,:,:,:,:,2,4) = -(-oddSlow + evenFast);
  yd(tt+1,:,:,:,:,:,:) = max(vd(tt+1,:,:,:,:,:,:),0).^2;
  rootYd(tt+1,:,:,:,:,:,:) = sqrt(yd(tt+1,:,:,:,:,:,:));
end
close(h);

return
