%% Parameters

% Spatial and temporal sampling
deltaX = 1/128; % deg of visual angle
preDuration = 10; % msec
duration = 250; % msec
postDuration = 500; % msec
totalDuration = duration+postDuration;
va = 0.125; % visual angle in deg

% Stimulus parameters
contrast = 1.0;
% contrast = 0.1;

% *** Redefine parameters for debugging ***
tauV = 1; tauA = 1; tauU = 10; deltaT = 1;

%% Make stimulus

% tIm, xi1Im, and xi2Im are each of size T x yDim x xDim
% xi1Im is y dimension
% xi2Im is x dimension
t = [-preDuration:deltaT:totalDuration-deltaT]'; % msec
T = length(t);
xi1 = [-va:deltaX:va-deltaX]; % deg
xi2 = [-va:deltaX:va-deltaX]; % deg
[xi1Im,tIm,xi2Im] = meshgrid(xi2,t,xi1);
dims = [size(tIm,2) size(tIm,3)];

% Drifting grating, contrast-reversing grating, flickering grating, or
% static grating
if strcmp(stimulusType,'static')
  x = cos(2*pi*stimulusSF*xi2Im + 2*pi*0*tIm);
elseif strcmp(stimulusType,'leftward')
  x = cos(2*pi*stimulusSF*xi2Im + 2*pi*stimulusTF*tIm);
elseif strcmp(stimulusType,'upward')
  x = cos(2*pi*stimulusSF*xi1Im + 2*pi*stimulusTF*tIm);
elseif strcmp(stimulusType,'contrastReversing')
  x = cos(2*pi*stimulusSF*xi2Im) .* cos(2*pi*stimulusTF*tIm);
elseif strcmp(stimulusType,'flickering')
  x = cos(2*pi*stimulusSF*xi2Im) .* (0.5*(1+cos(2*pi*stimulusTF*tIm)));
elseif strcmp(stimulusType,'impulse')
  x = zeros(size(tIm));
  x(xi1Im==0 & xi2Im==0 & tIm==0) = 1;
end
x(tIm<0) = 0;
x(tIm>duration) = 0;
x = contrast * x;

% Uncomment this to show stimulus animation
% figure(1), clf;
% for tt=1:round(4/deltaT):T
%   imshow(x(tt,:,:),[-1 1],'InitialMagnification','fit');
%   drawnow;
% end

figure(2), clf;
imshow(squeeze(x(:,xi1==0,:)),[],'InitialMagnification','fit');
title('Stimulus (x)');

%% Temporal prefilter

% Temporal prefilter
xPrefiltered = squeeze(recurrentQuadratureFilter(x, lambdaX, omegaX, tauX, nFiltX, RIC, deltaT));
xPrefiltered = xPrefiltered(:,:,:,nFiltX);
xPrefilteredAmp = abs(xPrefiltered);

figure(3), clf;
if isreal(xPrefiltered)
    plot(t,squeeze(xPrefiltered(:,xi1==0,xi2==0)));
else
  plot(t,[real(squeeze(xPrefiltered(:,xi1==0,xi2==0))) imag(squeeze(xPrefiltered(:,xi1==0,xi2==0)))]);
  hold on;
  plot(t,squeeze(xPrefilteredAmp(:,xi2==0,xi1==0)), 'k');
  hold off;
end
xlim([-preDuration duration+postDuration]);
title('Temporal prefilter responses (zt)');
drawPublishAxis;

%% Recurrent normalization

% Run it
[z, y, v, yd, vd, n, a, u] = organicNormalizationFull(xPrefiltered, numOrientations, bandwidth, tauV, tauA, tauU, omega, b0, sigma, deltaT);

%% Plot responses for selected spatial frequency and orientation bands

% yPlot = y; vPlot = v;
yPlot = yd; vPlot = vd;

ySum = sum(yPlot,7);
ySumAll = mean(mean(sum(sum(sum(sum(yPlot,7),6),5),4),3),2);
vSumAll = mean(mean(sum(sum(sum(sum(vPlot,7),6),5),4),3),2);
maxEnergy = max(ySum(:));
maxAmp = sqrt(maxEnergy);

figure(4); clf;
xi1Ind = find(xi1 == 0);
xi2Ind = find(xi2 == 0);
if strcmp(stimulusType,'upward')
  ori = 4;
else
  ori = 1;
end
if strcmp(stimulusType,'impulse')
  lev = 1;
else
  lev = round(1/(4*deltaX)/stimulusSF);
end

subplot(5,1,1);
plot(t,ySumAll);
xlim([-preDuration totalDuration]);
set(gca,'xtick',[0:duration:totalDuration]);
title('Sum of normalized responses (y)');
drawPublishAxis;

subplot(5,1,2);
plot(t,squeeze(yPlot(:,xi2Ind,xi1Ind,lev,ori,1,:)));
hold on
plot(t,ySum(:,xi2Ind,xi1Ind,lev,ori,1),'k');
hold off
xlim([-preDuration totalDuration]);
set(gca,'xtick',[0:duration:totalDuration]);
ylim([0 maxEnergy]);
title('Normalized responses (y)');
drawPublishAxis;

subplot(5,1,3);
plot(t,squeeze(yPlot(:,xi2Ind,xi1Ind,lev,ori,2,:)));
hold on
plot(t,ySum(:,xi2Ind,xi1Ind,lev,ori,2),'k');
hold off
xlim([-preDuration totalDuration]);
set(gca,'xtick',[0:duration:totalDuration]);
ylim([0 maxEnergy]);
drawPublishAxis;

subplot(5,1,4);
plot(t,squeeze(vPlot(:,xi2Ind,xi1Ind,lev,ori,1,:)));
xlim([-preDuration totalDuration]);
set(gca,'xtick',[0:duration:totalDuration]);
ylim([-maxAmp maxAmp]);
title('Membrane potential responses (v)');
drawPublishAxis;

subplot(5,1,5);
plot(t,squeeze(vPlot(:,xi2Ind,xi1Ind,lev,ori,2,:)));
xlim([-preDuration totalDuration]);
set(gca,'xtick',[0:duration:totalDuration]);
ylim([-maxAmp maxAmp]);
drawPublishAxis;

print([figurePath,'/responses tauU=',num2str(tauU),'.eps'],'-depsc','-painters');

%% Plot all normalized responses 
% Each panel correspondes to a spatial frequency, orientation, and
% direction band. Plot both normalized half-squared responses and the
% normalized energies.

numLevels = maxLevel(dims,bandwidth);

figure(5); clf;
dir = 1;
for lev = 1:numLevels
  for ori = 1:numOrientations
    whichPlot = numOrientations*(lev-1) + (ori-1) + 1;
    subplot(numLevels,numOrientations,whichPlot);
    plot(t,squeeze(ySum(:,xi2Ind,xi1Ind,lev,ori,dir,:)),'k');
    hold on;
    plot(t,squeeze(yPlot(:,xi2Ind,xi1Ind,lev,ori,dir,:)));
    hold off;
    xlim([-preDuration totalDuration]);
    set(gca,'xtick',[0:duration:totalDuration]);
    ylim([0 maxEnergy]);
    if (whichPlot == 6)
      title('Normalized responses (y)');
    end
    drawPublishAxis;
  end
end
print([figurePath,'/responsesAll1 tauU=',num2str(tauU),'.eps'],'-depsc','-painters');

figure(6); clf;
dir = 2;
for lev = 1:numLevels
  for ori = 1:numOrientations
    whichPlot = numOrientations*(lev-1) + (ori-1) + 1;
    subplot(numLevels,numOrientations,whichPlot);
    plot(t,squeeze(ySum(:,xi2Ind,xi1Ind,lev,ori,dir,:)),'k');
    hold on;
    plot(t,squeeze(yPlot(:,xi2Ind,xi1Ind,lev,ori,dir,:)));
    hold off;
    xlim([-preDuration totalDuration]);
    set(gca,'xtick',[0:duration:totalDuration]);
    ylim([0 maxEnergy]);
    if (whichPlot == 6)
      title('Normalized responses (y)');
    end
    drawPublishAxis;
  end
end
print([figurePath,'/responsesAll2 tauU=',num2str(tauU),'.eps'],'-depsc','-painters');