%% Make RFs (orientation tuning curves) and input stimulus

%%%%%%%%%%%%
% Make RFs %
%%%%%%%%%%%%

% RFs are raised cosines
% Tuning curves are raised cosines
% Spatial RFs are raised cosines
% Recurrent weights are QMFs
% Spatial normalization weights are raised cosines (4x wider than RFs)
% Orientation normalization weights are 1s
[theta, deltaTheta, thetaPreferred, thetaRFs] = makeRFs(Ntheta, overlapTheta, 1, [-1 1]);

% Number of stimulus samples
Mtheta = length(theta);

%%%%%%%%%%%%%%%%%%%%%%%%%
% Normalization weights %
%%%%%%%%%%%%%%%%%%%%%%%%%

% Normalization weights are an NxN matrix. Comment/uncomment below to pick
% one of the options.

% Option 1: sum evenly over all orientations
W = ones(Ntheta,Ntheta);

% Option 2: random symmetric weight matrix.
% tmp = rand(Ntheta,Ntheta);
% W = tmp' * tmp;
% W = W / (mean(mean(W)));

% Option 3: random  weight matrix.
% W = rand(Ntheta,Ntheta);
% W = W / (mean(mean(W)));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Make full contrast input stimulus and compute input drive %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Input stimulus is a vector, each element is the contrast of an
% orientation component. The sum of the elements in the vector equals 1 at
% full contrast. Three different options for the stimulus pattern: single
% orientation component, random pair of orientations, random combination of
% all orientations.
% Option 1: single orientation component
% Option 2: orthogonal pair of orientations
% Option 3: random combination of orientations
s = zeros(Mtheta,1);
if strcmp(stimulusPattern,'grating')
  s(floor(Mtheta/2)+1) = 1;
elseif strcmp(stimulusPattern,'plaid')
  s(1) = 0.5;
  s(floor(Mtheta/2)+1) = 0.5;
elseif strcmp(stimulusPattern,'noise')
  s = (1/Ntheta) * rand(Mtheta,1);
  s = rand(Mtheta,1);
end
% Make sure it doesn't exceed 100% contrast
s = s / sum(s);

% Input drive at full contrast
z1 = (thetaRFs * s);
totalEnergy = sum(z1.^2)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Plot input stimulus, tuning curves, and input drive %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

figure(1); clf;

subplot(3,1,1)
plot(theta*180/pi, min(s,1));
xlim([-90 90]);
set(gca,'xtick',[-90:30:90]);
ylim([0 1]);
set(gca,'ytick',[0:0.5:1]);
xlabel('Orientation');
title('Input stimulus');
drawPublishAxis

% Rescale spatial RFs for plotting
sumRFs = sqrt(max(sum(thetaRFs.^2)));
scaledThetaRFs = thetaRFs / sumRFs;

subplot(3,1,2)
plot(theta*180/pi, [scaledThetaRFs' sum(scaledThetaRFs.^2)']);
xlim([-90 90]);
set(gca,'xtick',[-90:30:90]);
ylim([0 1]);
set(gca,'ytick',[0:0.5:1]);
xlabel('Orientation');
title('Tuning curves');
drawPublishAxis

subplot(3,1,3)
plot(1:Ntheta,z1);
xlim([1 Ntheta+1]);
set(gca,'xtick',[1:Ntheta]);
%ylim([0 1]);
%set(gca,'ytick',[0:0.5:1]);
xlabel('Neuron');
title('Input drive');
drawPublishAxis

print([figurePath,'/tuningCurves.eps'],'-depsc','-painters');

%% Steady state

% Contrasts
cStep = 0.025;
c = 10.^[-2:cStep:0];
nC = length(c);

% Input drive matrix: multiply z1 (input drive at full contrast) with each
% of the contrasts.
z = z1 * c;
zSqr = z.^2;

% Feedforward responses
yFeedforward = zSqr ./ (W * zSqr + sigma^2);

% Modulators
a = zeros(size(yFeedforward));
u = zeros(size(yFeedforward));
I = eye(Ntheta);
for cc = 1:nC
  yTmp = yFeedforward(:,cc);
  u(:,cc) = ((sigma * b0)/(1+b0))^2 * inv(I-W*diag(yTmp))*ones(Ntheta,1);
  a(:,cc) = sqrt(u(:,cc)) ./ (1-sqrt(u(:,cc)));
end

% Effective gain
ge = 1 ./ (W * zSqr + sigma^2);
geMax = 1/sigma^2;

% Effective time constant
tauE = tauV * ((1+b0)/b0) * sqrt(ge);
tauEmax = tauV * ((1+b0)/b0) * sqrt(geMax);

% Steady state responses
y = ge .* zSqr;
shouldBeZero = max(max(y-yFeedforward))
ySum = sum(y);
R50 = ySum(floor(length(c)/2)+1)

% Plot it
figure(2); clf;

subplot(6,1,1);
% yScale = max(y(:));
yScale = 0.4;
semilogx(c,y');
ylim([0 yScale]);
set(gca,'ytick',[0:yScale/2:yScale]);
xlabel('Contrast');
title('Steady state responses: y');
drawPublishAxis

subplot(6,1,2);
% ySumScale = max(ySum);
ySumScale = 1;
semilogx(c,ySum,'k');
% hold on
% semilogx(c,sum(yFeedforward)','o');
% hold off
ylim([0 ySumScale]);
set(gca,'ytick',[0:ySumScale/2:ySumScale]);
xlabel('Contrast');
title('Steady state responses: sum of y');
drawPublishAxis

subplot(6,1,3);
semilogx(c,[mean(a)' sqrt(mean(u)')]);
xlabel('Contrast');
title('Modulators: a and sqrt(u)');
drawPublishAxis

subplot(6,1,4);
semilogx(c,ge','k');
ylim([0 geMax]);
xlabel('Contrast');
title('Effective gain: ge');
drawPublishAxis

subplot(6,1,5);
semilogx(c,tauE','k');
ylim([0 tauEmax]);
set(gca,'ytick',[0:tauEmax/2:tauEmax]);
xlabel('Contrast');
title('Effective time constant: taue');
drawPublishAxis

subplot(6,1,6);
%plot(sum(y),sum(a),'.');
%title('Modulator vs response: sum(a) vs sum(y)');
plot(y(:),a(:),'.');
title('Modulator vs response: a vs y');
drawPublishAxis

print([figurePath,'/steadyState.eps'],'-depsc','-painters');

%% Dynamics of responses with increasing contrast

% Attentional gain: optionally boost input drive from neuron tuned to
% vertical (90 degrees).
attentionGain = 1;
%attentionGain = 1.2;
b = b0 * ones(1,Ntheta);
attendID = floor(Ntheta/2)+1;
b(attendID) = attentionGain * b0;

% Stimulus timing (msec)
totalDuration = 2 * targetDuration * length(contrasts);
t = [-preDuration:deltaT:totalDuration-deltaT]';
T = length(t);

% Input
% x0 is sequence of increasing contrasts (modulator)
% x is multiplied by (co-)sinusoidal modulation (carrier)
x0 = zeros(T,1);
for ii = 1:length(contrasts)
  targetOnsetTime = 2*(ii-1) * targetDuration;
  targetOffsetTime = (2*(ii-1)+1) * targetDuration;
  id = (t>targetOnsetTime) & (t<targetOffsetTime);
  x0(id) = contrasts(ii);
end
if strcmp(stimulusType,'static')
  x = x0;
else
  x = x0 .* (cos(2*pi*stimulusTF*t) + i*sin(2*pi*stimulusTF*t));
end

% Temporal prefilter
xTmp = zeros(T,1,1);
xTmp(:) = x;
xPrefiltered = squeeze(recurrentQuadratureFilter(xTmp, lambdaX, omegaX, tauX, nFiltX, RIC, deltaT));
xPrefiltered = xPrefiltered(:,nFiltX);

% Input drive: multiply by z1 (input drive at full contrast), split into 4
% phases from the real and imaginary parts
z = zeros(T,Ntheta,4);
z(:,:,1) = real(xPrefiltered) * z1';
z(:,:,2) = -real(xPrefiltered) * z1';
z(:,:,3) = imag(xPrefiltered) * z1';
z(:,:,4) = -imag(xPrefiltered) * z1';

%%%%%%%%%%
% Run it %
%%%%%%%%%%

[y, v, v1, a, u] = organicNormalizationOri(z, W, tauV, tauA, tauU, omega, b, b0, sigma, deltaT);
ySum = sum(y,3);

% Compute effective feedforward gain (ignoring attention gain)
ge = ((1+a)./a).^2 * (b0/(1+b0))^2;

% Compute steady state responses (ignoring attention gain) for comparison
zSteadyState = contrasts'*z1';
ySteadyState = (zSteadyState.^2) ./ ((zSteadyState.^2)*W' + sigma^2);
ySteadyStateSum = sum(ySteadyState,2);

% Modulators
aSteadyState = zeros(size(zSteadyState));
uSteadyState = zeros(size(zSteadyState));
I = eye(Ntheta);
for cc = 1:length(contrasts)
  yTmp = ySteadyState(cc,:)';
  uSteadyState(cc,:) = ((sigma * b0)/(1+b0))^2 * inv(I-W*diag(yTmp))*ones(Ntheta,1);
  aSteadyState(cc,:) = sqrt(uSteadyState(cc,:)) ./ (1-sqrt(uSteadyState(cc,:)));
end

%%
%%%%%%%%%%%
% Plot it %
%%%%%%%%%%%

% Subsample time for plotting
tSkip = floor(deltaTPlot / deltaT);

figure(3); clf;

zScale = max(max(zSteadyState));
% yScale = 2 * attentionGain * max(max(ySteadyState));
yScale = 1;
% ySumScale = 2 * attentionGain * max(ySteadyStateSum);
ySumScale = 2;
aScale = 1;
uScale = 1;

subplot(4,1,1);
zAmp = sqrt(z(:,:,1).^2 + z(:,:,3).^2);
plot(t(1:tSkip:end),[z(1:tSkip:end,:,1)]);
% plot(t(1:tSkip:end),[z(1:tSkip:end,:,1) zAmp]);
% plot(t(1:tSkip:end),[z(1:tSkip:end,:,1) z(1:tSkip:end,:,2) z(1:tSkip:end,:,3) z(1:tSkip:end,:,4)]);
hold on
for cc = 1:length(contrasts);
  targetTime = (2*(cc-1)+1) * targetDuration;
  plot(repmat(targetTime,2,1),repmat(zSteadyState(cc,:),2,1),'o');
end
hold off
xlim([-preDuration,totalDuration]);
ylim([-zScale zScale]);
xlabel('Time (msec)');
title('Input drive: z');
drawPublishAxis

subplot(4,1,2);
plot(t(1:tSkip:end),y(1:tSkip:end,:,1));
% plot(t(1:tSkip:end),[y(1:tSkip:end,:,1) y(1:tSkip:end,:,2) y(1:tSkip:end,:,3) y(1:tSkip:end,:,4)]);
hold on
for cc = 1:length(contrasts);
  targetTime = (2*(cc-1)+1) * targetDuration;
  plot(repmat(targetTime,2,1),repmat(ySteadyState(cc,:),2,1),'o');
end
hold off
xlim([-preDuration,totalDuration]);
ylim([0 yScale]);
set(gca,'ytick',[0:yScale/2:yScale]);
set(gca,'Clipping','off');
xlabel('Time (msec)');
title('Responses: simple cells');
drawPublishAxis

subplot(4,1,3);
plot(t(1:tSkip:end),ySum(1:tSkip:end,:));
hold on
for cc = 1:length(contrasts);
  targetTime = (2*(cc-1)+1) * targetDuration;
  plot(repmat(targetTime,2,1),repmat(ySteadyState(cc,:),2,1),'o');
end
hold off
xlim([-preDuration,totalDuration]);
ylim([0 yScale]);
set(gca,'ytick',[0:yScale/2:yScale]);
set(gca,'Clipping','off');
xlabel('Time (msec)');
title('Responses: complex cells');
drawPublishAxis

subplot(4,1,4);
ySumAll = sum(ySum,2);
plot(t(1:tSkip:end),ySumAll(1:tSkip:end));
hold on
for cc = 1:length(contrasts);
  targetTime = (2*(cc-1)+1) * targetDuration;
  plot(targetTime,ySteadyStateSum(cc),'o','Color',defaultColors(1,:));
end
plot(t,0.5*ones(size(t)),'k-.');
plot(t,ones(size(t)),'k-.');
hold off
xlim([-preDuration,totalDuration]);
ylim([0 ySumScale]);
set(gca,'ytick',[0:ySumScale/2:ySumScale]);
set(gca,'Clipping','off');
xlabel('Time (msec)');
title('Summed responses');
drawPublishAxis

print([figurePath,'/dynamics1 tauU=',num2str(tauU),'.eps'],'-depsc','-painters');

figure(4); clf;

subplot(5,1,1);
plot(t(1:tSkip:end),a(1:tSkip:end,:));
xlim([-preDuration,totalDuration]);
ylim([0 aScale]);
set(gca,'ytick',[0:aScale/2:aScale]);
set(gca,'Clipping','off');
xlabel('Time (msec)');
title('Modulator: a');
drawPublishAxis

subplot(5,1,2);
plot(t(1:tSkip:end),sqrt(u(1:tSkip:end,:)));
xlim([-preDuration,totalDuration]);
ylim([0 uScale]);
set(gca,'ytick',[0:uScale/2:uScale]);
set(gca,'Clipping','off');
xlabel('Time (msec)');
title('Modulator: sqrt(u)');
drawPublishAxis

subplot(5,1,3);
semilogy(t(1:tSkip:end),ge(1:tSkip:end,:));
xlim([-preDuration,totalDuration]);
ylim([0.01, 100]);
set(gca,'Clipping','off');
xlabel('Time (msec)');
title('Effective gain: ge');
drawPublishAxis

subplot(5,1,4);
plot(t(1:tSkip:end),v(1:tSkip:end,:,1));
xlim([-preDuration,totalDuration]);
ylim([-sqrt(yScale) sqrt(yScale)]);
set(gca,'Clipping','off');
xlabel('Time (msec)');
title('Membrane potential responses: v');
drawPublishAxis

subplot(5,1,5);
plot(t(1:tSkip:end),v1(1:tSkip:end,:,1));
xlim([-preDuration,totalDuration]);
ylim(b0*[-zScale zScale]);
set(gca,'Clipping','off');
xlabel('Time (msec)');
title('Membrane potential responses w/ feedback disabled');
drawPublishAxis

print([figurePath,'/dynamics2 tauU=',num2str(tauU),'.eps'],'-depsc','-painters');

figure(5); clf;
aSum = sum(a,2);
uSum = sum(sqrt(u),2);
aSumScale = max(sum(aSteadyState,2));
uSumScale = max(sum(sqrt(uSteadyState),2));
ySumScale = max(ySteadyStateSum);
plot(t(1:tSkip:end),[ySumAll(1:tSkip:end)/ySumScale aSum(1:tSkip:end)/aSumScale uSum(1:tSkip:end)/uSumScale]);
hold on
plot(t,ones(size(t)),'k-.');
hold off
xlim([2*targetDuration*(length(contrasts)-1),2*targetDuration*length(contrasts)]);
ylim([0 6]);
% set(gca,'Clipping','off');
xlabel('Time (msec)');
title('Relative response phase of y, a, and u');
drawPublishAxis

print([figurePath,'/phase alignment tauU=',num2str(tauU),'.eps'],'-depsc','-painters');

%%

yWeightedSum = ySum * W';
yRatios = ySum ./ (repmat(ySumAll,[1,Ntheta]) + 1e-16);
% yRatios = ySum ./ (yWeightedSum + 1e-16);

figure(6); clf;
plot(t(1:tSkip:end),yRatios(1:tSkip:end,:));
xlim([-preDuration,totalDuration]);
%ylim([0 yScale]);
%set(gca,'ytick',[0:yScale/2:yScale]);
set(gca,'Clipping','off');
xlabel('Time (msec)');
title('Responses: complex cells');
drawPublishAxis

print([figurePath,'/response ratios tauU=',num2str(tauU),'.eps'],'-depsc','-painters');
